@@ -1,6 +1,6 @@
 /*  vcfnorm.c -- Left-align and normalize indels.
 
-    Copyright (C) 2013-2016 Genome Research Ltd.
+    Copyright (C) 2013-2017 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -34,6 +34,7 @@ THE SOFTWARE.  */
 #include <htslib/vcf.h>
 #include <htslib/synced_bcf_reader.h>
 #include <htslib/faidx.h>
+#include <htslib/khash_str2int.h>
 #include "bcftools.h"
 #include "rbuf.h"
 
@@ -53,6 +54,15 @@ typedef struct
 }
 map_t;
 
+// primitive comparison of two records' alleles via hashes; normalized alleles assumed
+typedef struct
+{
+    int n;  // number of alleles
+    char *ref, *alt;
+    void *hash;
+}
+cmpals_t;
+
 typedef struct
 {
     char *tseq, *seq;
@@ -72,6 +82,8 @@ typedef struct
     int aln_win;            // the realignment window size (maximum repeat size)
     bcf_srs_t *files;       // using the synced reader only for -r option
     bcf_hdr_t *hdr;
+    cmpals_t *cmpals;
+    int ncmpals, mcmpals;
     faidx_t *fai;
     struct { int tot, set, swap; } nref;
     char **argv, *output_fname, *ref_fname, *vcf_fname, *region, *targets;
@@ -94,7 +106,7 @@ static inline int replace_iupac_codes(char *seq, int nseq)
 }
 static inline int has_non_acgtn(char *seq, int nseq)
 {
-    char *end = nseq ? seq + nseq : seq + UINT32_MAX;   // arbitrary large number
+    char *end = seq + nseq;
     while ( *seq && seq<end )
     {
         char c = toupper(*seq);
@@ -326,7 +338,7 @@ static int realign(args_t *args, bcf1_t *line)
     {
         if ( line->d.allele[i][0]=='<' ) return ERR_SYMBOLIC;  // symbolic allele
         if ( line->d.allele[i][0]=='*' ) return ERR_SPANNING_DELETION;  // spanning deletion
-        if ( has_non_acgtn(line->d.allele[i],0) )
+        if ( has_non_acgtn(line->d.allele[i],line->shared.l) )
         {
             if ( args->check_ref==CHECK_REF_EXIT )
                 error("Non-ACGTN alternate allele at %s:%d .. REF_SEQ:'%s' vs VCF:'%s'\n", bcf_seqname(args->hdr,line),line->pos+1,ref,line->d.allele[i]);
@@ -1510,10 +1522,74 @@ static bcf1_t *mrows_flush(args_t *args)
     }
     return NULL;
 }
+static void cmpals_add(args_t *args, bcf1_t *rec)
+{
+    args->ncmpals++;
+    hts_expand0(cmpals_t, args->ncmpals, args->mcmpals, args->cmpals);
+    cmpals_t *cmpals = args->cmpals + args->ncmpals - 1;
+    free(cmpals->ref);
+    cmpals->ref = strdup(rec->d.allele[0]);
+    cmpals->n   = rec->n_allele;
+    if ( rec->n_allele==2 )
+    {
+        free(cmpals->alt);
+        cmpals->alt = strdup(rec->d.allele[1]);
+    }
+    else
+    {
+        if ( cmpals->hash ) khash_str2int_destroy_free(cmpals->hash);
+        cmpals->hash = khash_str2int_init();
+        int i;
+        for (i=1; i<rec->n_allele; i++)
+            khash_str2int_inc(cmpals->hash, strdup(rec->d.allele[i]));
+    }
+}
+static int cmpals_match(args_t *args, bcf1_t *rec)
+{
+    int i, j;
+    for (i=0; i<args->ncmpals; i++)
+    {
+        cmpals_t *cmpals = args->cmpals + i;
+        if ( rec->n_allele != cmpals->n ) continue;
+
+        // NB. assuming both are normalized
+        if ( strcmp(rec->d.allele[0], cmpals->ref) ) continue;
+
+        // the most frequent case
+        if ( rec->n_allele==2 )
+        {
+            if ( strcmp(rec->d.allele[1], cmpals->alt) ) continue;
+            return 1;
+        }
+
+        khash_t(str2int) *hash = (khash_t(str2int)*) cmpals->hash;
+        for (j=1; j<rec->n_allele; j++) 
+            if ( !khash_str2int_has_key(hash, rec->d.allele[j]) ) break;
+        if ( j<rec->n_allele ) continue;
+        return 1;
+    }
+    cmpals_add(args, rec);
+    return 0;
+}
+static void cmpals_reset(args_t *args) { args->ncmpals = 0; }
+static void cmpals_destroy(args_t *args)
+{
+    int i;
+    for (i=0; i<args->mcmpals; i++)
+    {
+        cmpals_t *cmpals = args->cmpals + i;
+        free(cmpals->ref);
+        free(cmpals->alt);
+        if ( cmpals->hash ) khash_str2int_destroy_free(cmpals->hash);
+    }
+    free(args->cmpals);
+}
+
 static void flush_buffer(args_t *args, htsFile *file, int n)
 {
     bcf1_t *line;
     int i, k;
+    int prev_rid = -1, prev_pos = -1, prev_type = 0;
     for (i=0; i<n; i++)
     {
         k = rbuf_shift(&args->rbuf);
@@ -1534,6 +1610,26 @@ static void flush_buffer(args_t *args, htsFile *file, int n)
                 continue;
             }
         }
+        else if ( args->rmdup )
+        {
+            int line_type = bcf_get_variant_types(args->lines[k]);
+            if ( prev_rid>=0 && prev_rid==args->lines[k]->rid && prev_pos==args->lines[k]->pos )
+            {
+                if ( args->rmdup & BCF_SR_PAIR_ANY ) continue;    // rmdup by position only
+                if ( args->rmdup & BCF_SR_PAIR_SNPS && line_type&(VCF_SNP|VCF_MNP) && prev_type&(VCF_SNP|VCF_MNP) ) continue;
+                if ( args->rmdup & BCF_SR_PAIR_INDELS && line_type&(VCF_INDEL) && prev_type&(VCF_INDEL) ) continue;
+                if ( args->rmdup & BCF_SR_PAIR_EXACT && cmpals_match(args, args->lines[k]) ) continue;
+            }
+            else
+            {
+                prev_rid  = args->lines[k]->rid;
+                prev_pos  = args->lines[k]->pos;
+                prev_type = 0;
+                if ( args->rmdup & BCF_SR_PAIR_EXACT ) cmpals_reset(args);
+            }
+            prev_type |= line_type;
+            if ( args->rmdup & BCF_SR_PAIR_EXACT ) cmpals_add(args, args->lines[k]);
+        }
         bcf_write1(file, args->hdr, args->lines[k]);
     }
     if ( args->mrows_op==MROWS_MERGE && !args->rbuf.n )
@@ -1562,6 +1658,7 @@ static void init_data(args_t *args)
 
 static void destroy_data(args_t *args)
 {
+    cmpals_destroy(args);
     int i;
     for (i=0; i<args->rbuf.m; i++)
         if ( args->lines[i] ) bcf_destroy1(args->lines[i]);
@@ -1658,17 +1755,20 @@ static void normalize_vcf(args_t *args)
             int line_type = bcf_get_variant_types(line);
             if ( prev_rid>=0 && prev_rid==line->rid && prev_pos==line->pos )
             {
-                if ( (args->rmdup>>1)&COLLAPSE_ANY ) continue;
-                if ( (args->rmdup>>1)&COLLAPSE_SNPS && line_type&(VCF_SNP|VCF_MNP) && prev_type&(VCF_SNP|VCF_MNP) ) continue;
-                if ( (args->rmdup>>1)&COLLAPSE_INDELS && line_type&(VCF_INDEL) && prev_type&(VCF_INDEL) ) continue;
+                if ( args->rmdup & BCF_SR_PAIR_ANY ) continue;    // rmdup by position only
+                if ( args->rmdup & BCF_SR_PAIR_SNPS && line_type&(VCF_SNP|VCF_MNP) && prev_type&(VCF_SNP|VCF_MNP) ) continue;
+                if ( args->rmdup & BCF_SR_PAIR_INDELS && line_type&(VCF_INDEL) && prev_type&(VCF_INDEL) ) continue;
+                if ( args->rmdup & BCF_SR_PAIR_EXACT && cmpals_match(args, line) ) continue;
             }
             else
             {
                 prev_rid  = line->rid;
                 prev_pos  = line->pos;
                 prev_type = 0;
+                if ( args->rmdup & BCF_SR_PAIR_EXACT ) cmpals_reset(args);
             }
             prev_type |= line_type;
+            if ( args->rmdup & BCF_SR_PAIR_EXACT ) cmpals_add(args, line);
         }
 
         // still on the same chromosome?
@@ -1725,7 +1825,7 @@ static void usage(void)
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "    -c, --check-ref <e|w|x|s>         check REF alleles and exit (e), warn (w), exclude (x), or set (s) bad sites [e]\n");
     fprintf(stderr, "    -D, --remove-duplicates           remove duplicate lines of the same type.\n");
-    fprintf(stderr, "    -d, --rm-dup <type>               remove duplicate snps|indels|both|any\n");
+    fprintf(stderr, "    -d, --rm-dup <type>               remove duplicate snps|indels|both|all|none\n");
     fprintf(stderr, "    -f, --fasta-ref <file>            reference sequence (MANDATORY)\n");
     fprintf(stderr, "    -m, --multiallelics <-|+>[type]   split multiallelics (-) or join biallelics (+), type: snps|indels|both|any [both]\n");
     fprintf(stderr, "        --no-version                  do not append version and command line to the header\n");
@@ -1786,10 +1886,12 @@ int main_vcfnorm(int argc, char *argv[])
         switch (c) {
             case 'N': args->do_indels = 0; break;
             case 'd':
-                if ( !strcmp("snps",optarg) ) args->rmdup = COLLAPSE_SNPS<<1;
-                else if ( !strcmp("indels",optarg) ) args->rmdup = COLLAPSE_INDELS<<1;
-                else if ( !strcmp("both",optarg) ) args->rmdup = COLLAPSE_BOTH<<1;
-                else if ( !strcmp("any",optarg) ) args->rmdup = COLLAPSE_ANY<<1;
+                if ( !strcmp("snps",optarg) ) args->rmdup = BCF_SR_PAIR_SNPS;
+                else if ( !strcmp("indels",optarg) ) args->rmdup = BCF_SR_PAIR_INDELS;
+                else if ( !strcmp("both",optarg) ) args->rmdup = BCF_SR_PAIR_BOTH;
+                else if ( !strcmp("all",optarg) ) args->rmdup = BCF_SR_PAIR_ANY;
+                else if ( !strcmp("any",optarg) ) args->rmdup = BCF_SR_PAIR_ANY;
+                else if ( !strcmp("none",optarg) ) args->rmdup = BCF_SR_PAIR_EXACT;
                 else error("The argument to -d not recognised: %s\n", optarg);
                 break;
             case 'm':
@@ -1822,8 +1924,8 @@ int main_vcfnorm(int argc, char *argv[])
                 break;
             case 'o': args->output_fname = optarg; break;
             case 'D':
-                fprintf(stderr,"Warning: `-D` is functional but deprecated, replaced by `-d both`.\n"); 
-                args->rmdup = COLLAPSE_NONE<<1;
+                fprintf(stderr,"Warning: `-D` is functional but deprecated, replaced by and alias of `-d none`.\n"); 
+                args->rmdup = BCF_SR_PAIR_EXACT;
                 break;
             case 's': args->strict_filter = 1; break;
             case 'f': args->ref_fname = optarg; break;
@@ -1842,9 +1944,7 @@ int main_vcfnorm(int argc, char *argv[])
             default: error("Unknown argument: %s\n", optarg);
         }
     }
-    if ( argc>optind+1 ) usage();
-    if ( !args->ref_fname && !args->mrows_op && !args->rmdup ) usage();
-    if ( !args->ref_fname && args->check_ref&CHECK_REF_FIX ) error("Expected --fasta-ref with --check-ref s\n");
+
     char *fname = NULL;
     if ( optind>=argc )
     {
@@ -1853,6 +1953,9 @@ int main_vcfnorm(int argc, char *argv[])
     }
     else fname = argv[optind];
 
+    if ( !args->ref_fname && !args->mrows_op && !args->rmdup ) error("Expected -f, -m, -D or -d option\n");
+    if ( !args->ref_fname && args->check_ref&CHECK_REF_FIX ) error("Expected --fasta-ref with --check-ref s\n");
+
     if ( args->region )
     {
         if ( bcf_sr_set_regions(args->files, args->region,region_is_file)<0 )
